package com.zehong.monitorreprotdatafrogovernmentnew.task;

import com.zehong.monitorreprotdatafrogovernmentnew.dao.jingye.JingyeDetectorInfoDao;
import com.zehong.monitorreprotdatafrogovernmentnew.dao.yingde.YingdeDetectorInfoDao;
import com.zehong.monitorreprotdatafrogovernmentnew.dao.zhengyuan.ZhengyuanDetectorInfoDao;
import com.zehong.monitorreprotdatafrogovernmentnew.entity.DetectorInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.annotation.Resource;
import java.util.Date;

@Configuration
@EnableScheduling
@PropertySource(value = "classpath:application.yml")
public class SaticScheduleTask {

    @Resource
    private JingyeDetectorInfoDao jingyeDetectorInfoDao;

    @Resource
    private YingdeDetectorInfoDao yingdeDetectorInfoDao;

    @Resource
    private ZhengyuanDetectorInfoDao zhengyuanDetectorInfoDao;

    @Autowired
    private SendSmsTask dynamicTask;

    @Value("${sendSmsCron}")
    private String sendSmsCron;

    @Scheduled(cron = "${cron}")
    private void configureTasks() {
        DetectorInfo jingye = jingyeDetectorInfoDao.getRecentSendData();
        if(null != jingye && null != jingye.getUpdateTime()){
            compareReportDate(jingye.getUpdateTime(),"jingye");
        }

        DetectorInfo yingde = yingdeDetectorInfoDao.getRecentSendData();
        if(null != yingde && null != yingde.getUpdateTime()){
            compareReportDate(yingde.getUpdateTime(),"yingde");
        }

        DetectorInfo zhengyuan = zhengyuanDetectorInfoDao.getRecentSendData();
        if(null != zhengyuan && null != zhengyuan.getUpdateTime()){
            compareReportDate(zhengyuan.getUpdateTime(),"zhengyuan");
        }

    }

    /**
     * 比较上报时间 超过五分钟为发送数据发送短信预警
     * @param reportDate 上报时间
     * @param company 公司
     */
    private void compareReportDate(Date reportDate,String company){
        long period = System.currentTimeMillis() - reportDate.getTime();
        //超过十五分钟未发送数据发送短信报警
        if(15 < period/(60 * 1000)){
            //如果超过四小时未推送，不再提醒
            if(isOverFourHour(period,company)){
                return;
            }
            //如未及时处理没半小时发送一次短信
            if(!dynamicTask.isExist(company)){
                //发送短信
                dynamicTask.sendSms(company);
                //启动定时半小时后重新发送
                dynamicTask.add(company,sendSmsCron);
            }

            return;
        }
        removeTask(company);
    }

    /**
     * 判断未推送时间，超过四小时不再提醒
     * @param period 未推送时间
     * @param company 公司名称
     */
    private boolean isOverFourHour(long period,String company){
        if(4 < period / (60 * 60 * 1000)){
            removeTask(company);
            return true;
        }
        return false;
    }

    /**
     * 移除定时器
     * @param company 公司名称
     */
    private void removeTask(String company){
        if (dynamicTask.isExist(company)){
            dynamicTask.stop(company);
        }
    }
}
